﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DpDemo
{
    /// <summary>
    /// Entry point
    /// </summary>
    public class Program
    {
        /// <summary>
        /// FUNCTIONAL_MODULE_DESCRIPTION
        /// DP Demo application
        /// -------------------
        /// 
        /// The DPDemo application shows you how to use the PROFIBUS API
        /// to access the functionality of the PROFIBUS API.
        /// 
        /// The DPDemo application has an auto detection routine to identify DP slaves.
        /// In case this does not work properly you should define your DP slave configuration
        /// manually. The header file DPDemo.h contains the necessary information.
        /// 
        /// To build the application executable you have to create a makefile for a
        /// NT console application, which compiles all DPDemo files. You have to
        /// provide the include path to your PROFIBUS SDK accordingly.
        /// 
        /// 
        /// Program structure
        /// -----------------
        /// 
        /// The DPDemo application works with PROFIBUS API (PAPI), default with board 0
        /// 
        /// 
        /// The following hierarchichal structure with six threads is used:
        /// 
        /// 
        /// Run (DpDemo.CS):    - creates the keyboard polling thread
        /// |                   - creates the basic management thread (FMB)
        /// |                   - waits for the end of one of the created threads to quit the program
        /// |
        /// |-- KeyboardThread (DpDemo.CS):    - handles the keyboard input
        /// |                                 - terminates after keys q, x or ESC
        /// |
        /// |-- BasicManagementThread (FMBLayer.CS): - initializes all PROFIBUS protocols
        ///     |                                     (FMB_Set_Configuration, FMB_Set_Busparameter)
        ///     |                                   - creates the DP management thread
        ///     |                                   - handles FMB events (timeout on bus, etc.)
        ///     |
        ///     |-- DpManagementThread (DPLayer.CS): - initializes the DP master
        ///         |                                 (DP_Init_Master)
        ///         |                               - looks for DP slaves (preconfigured, auto detection)
        ///         |                                 (DP_Download_Loc)
        ///         |                               - starts DP operation
        ///         |                                 (DP_Act_Param_Loc (STOP, CLEAR, OPERATE)
        ///         |                               - creates the DP input and DP output threads
        ///         |                               - handles DP events
        ///         |                                 (DP_Get_Slave_Diag.ind)
        ///         |
        ///         |-- DpInputDataThread (DPLayer.CS):  - reads data from the slaves
        ///         |
        ///         |-- DpOutputDataThread (DPLayer.CS): - writes data to the slaves
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            DPDemo.Instance.Run(args);
        }
    }

}
