﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DpV1Demo
{
   /// <summary>
   /// Entry point
   /// </summary>
   public class Program
   {
      /// <summary>
      /// FUNCTIONAL_MODULE_DESCRIPTION
      /// DP Demo application
      /// -------------------
      /// 
      /// The DPDemo application shows you how to use the PROFIBUS API
      /// to access the functionality of the PROFIBUS API.
      /// 
      /// The DPDemo application has an auto detection routine to identify DP slaves.
      /// In case this does not work properly you should define your DP slave configuration
      /// manually. The header file DPDemo.h contains the necessary information.
      /// 
      /// To build the application executable you have to create a makefile for a
      /// NT console application, which compiles all DPDemo files. You have to
      /// provide the include path to your PROFIBUS SDK accordingly.
      /// 
      /// 
      /// Program structure
      /// -----------------
      /// 
      /// The DPV1Demo application works with PROFIBUS API (PAPI), default with board 0
      /// 
      /// 
      /// The following hierarchichal structure with six threads is used:
      /// 
      /// 
      /// Run (DpV1Demo.CS):  - creates the keyboard polling thread
      /// |                   - creates the basic management thread (FMB)
      /// |                   - waits for the end of one of the created threads to quit the program
      /// |
      /// |-- KeyboardThread (DpDemo.CS):   - handles the keyboard input
      /// |                                 - terminates after keys q, x or ESC
      /// |
      /// |-- BasicManagementThread (FMBLayer.CS): - initializes all PROFIBUS protocols
      ///     |                                     (FMB_Set_Configuration, FMB_Set_Busparameter)
      ///     |                                   - creates the DP management thread
      ///     |                                   - handles FMB events (timeout on bus, etc.)
      ///     |
      ///     |-- DpManagementThread (DPLayer.CS): - initializes the DP master
      ///         |                                 (DP_Init_Master)
      ///         |                               - looks for DP slaves (preconfigured, auto detection)
      ///         |                                 (DP_Download_Loc)
      ///         |                               - starts DP operation
      ///         |                                 (DP_Act_Param_Loc (STOP)
      ///         |                               - starts DP operation
      ///         |                               - DP_INITIATE  - open a connection
      ///         |                               - DP_READ      - read acyclic data
      ///         |                               - DP_WRITE     - write acyclic data
      ///         |                               - DP_ABORT     - close a connection
        /// </summary>
      static void Main(string[] args)
      {
         DpV1Demo.Instance.Run(args);
      }
   }
}
