﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using PapiWrapper;
using PapiWrapper.Common;
using PapiWrapper.Dps;

namespace DpsDemo
{
    class Program
    {
        private static CPapi _papi;
        private static Byte boardNumber;
        private static UInt32 readTimeout, writeTimeout;

        private static Thread _ResponseThread;


        static void Main(string[] args)
        {
            boardNumber = 0;
            readTimeout = 2000;
            writeTimeout = 0;

            C_PROFI_SERVICE_DESCR profiSdb        = new C_PROFI_SERVICE_DESCR();
            C_DPS_INIT_SLAVE_REQ  dpsInitSlaveReq = new C_DPS_INIT_SLAVE_REQ();

            if (!Init(boardNumber, readTimeout, writeTimeout))
            {
               Console.Read();
               return;
            }

            _ResponseThread = new Thread(ListenerThread);
            _ResponseThread.Start();

            // Init Slave
            profiSdb.Layer     = (byte)LayerIdentifier.Dps;
            profiSdb.Service   = (byte)DPSService.InitSlave;
            profiSdb.Primitive = (byte)ServicePrimitive.Req;

            dpsInitSlaveReq.SlaveAdd = 0x05;
            dpsInitSlaveReq.MinTsdr  = DPSConstants.DefaultMinTsdr;

            dpsInitSlaveReq.AutoCfgResponse = Constants.PbTrue;
            dpsInitSlaveReq.AutoPrmResponse = Constants.PbTrue;
            dpsInitSlaveReq.AutoStartupInputs = Constants.PbTrue;

            dpsInitSlaveReq.SyncModeSupported = Constants.PbTrue;
            dpsInitSlaveReq.FreezeModeSupported = Constants.PbTrue;
            dpsInitSlaveReq.SetSlaveAddSupported = Constants.PbTrue;

            dpsInitSlaveReq.MaxInputDataLen   = Global.DPS_DEFAULT_INPUT_LEN;
            dpsInitSlaveReq.MaxOutputDataLen  = Global.DPS_DEFAULT_OUTPUT_LEN;
            dpsInitSlaveReq.MaxCfgDataLen     = Global.DPS_DEFAULT_CFG_DATA_LEN;
            dpsInitSlaveReq.MaxUsrPrmDataLen  = Global.DPS_DEFAULT_USR_PRM_DATA_LEN;
            dpsInitSlaveReq.MaxExtDiagDataLen = Global.DPS_DEFAULT_EXT_DIAG_DATA_LEN;
            dpsInitSlaveReq.MaxAddressDataLen = Global.DPS_DEFAULT_ADDRESS_DATA_LEN;

            dpsInitSlaveReq.IdentNumber = 0x0B35;
            dpsInitSlaveReq.UserWatchdogTimeout = DPSConstants.UserWatchdogDisabled;

            dpsInitSlaveReq.Reserved[0] = 0;
            dpsInitSlaveReq.Reserved[1] = 0;
            dpsInitSlaveReq.Reserved[2] = 0;
            dpsInitSlaveReq.Reserved[3] = 0;

            dpsInitSlaveReq.CfgDataLen = (byte) Global.DPS_DEFAULT_CFG_DATA.Length;
            dpsInitSlaveReq.EnhancedInitDataLen = 0;

            dpsInitSlaveReq.CfgData = Global.DPS_DEFAULT_CFG_DATA;

            try
            {
                // Send Request
                _papi.SendServiceRequestResponse(boardNumber, profiSdb, dpsInitSlaveReq);
                Console.WriteLine("Request sent ...");
            }
            catch (CPapiException papiExc)
            {
                Console.WriteLine(string.Format("Papi Error: {0} {1}", papiExc.Error, papiExc.StackTrace));
            }

            // Out Data
            byte   outDataSize = 180;
            byte[] outData = new byte[outDataSize];

            // In Data
            byte[] inData = new byte[5] {0x01, 0x02, 0x03, 0x04, 0x05};

            // In Out Data
            byte state = 0;


            bool bRun = true;
            while (bRun)
            {
                try
                {
                    ConsoleKeyInfo key = Console.ReadKey();

                    if (ConsoleKey.S == key.Key)
                    {
                        Console.WriteLine("Get Status");
                        GetStatus();
                    }
                    if (ConsoleKey.Escape == key.Key)
                    {
                        bRun = false;
                    }

                    if (ConsoleKey.I == key.Key)
                    {
                        _papi.SetDpsInputData(boardNumber, inData, (byte)inData.Length, ref state);
                        Console.WriteLine("Data set --> State: " + state.ToString());
                    }

                    if (ConsoleKey.O == key.Key)
                    {
                        outDataSize = 180;
                        state    = 0;
                        _papi.GetDpsOutputData(boardNumber, ref outData, ref outDataSize, ref state);
                        Console.WriteLine("InputData: " + state.ToString());
                        for (ushort i = 0; i < outDataSize; i++)
                        {
                            Console.Write(inData[i].ToString() + " / ");
                        }
                    }

                }
                catch (CPapiException papiExc)
                {
                    Console.WriteLine(string.Format("Papi Error: {0} {1}", papiExc.Error, papiExc.StackTrace));
                }
            }

            _ResponseThread.Abort();

            Console.WriteLine("ENDE ...");
            End(boardNumber);
        }

        private static void GetStatus()
        {
            C_PROFI_SERVICE_DESCR profiSdb = new C_PROFI_SERVICE_DESCR();
            profiSdb.Layer = (byte)LayerIdentifier.Dps;
            profiSdb.Service = (byte)DPSService.GetStatus;
            profiSdb.Primitive = (byte)ServicePrimitive.Req;

            C_DPS_GET_STATUS_CON_IND getStatus = new C_DPS_GET_STATUS_CON_IND();

            try
            {
                _papi.SendServiceRequestResponse(boardNumber, profiSdb, getStatus);
                Console.WriteLine("Status Request sent ...");

            }
            catch (CPapiException papiExc)
            {
                Console.WriteLine(string.Format("Papi Error: {0} {1}", papiExc.Error, papiExc.StackTrace));
            }
        }


        public static bool Init(byte boardnr, UInt32 readTO, UInt32 writeTO)
        {
            try
            {
                if(_papi == null) _papi = new CPapi();
                _papi.Init(boardnr, readTO, 0);
                return true;
            }
            catch (CPapiException papiExc)
            {
                Console.WriteLine(string.Format("Papi Error: {0} {1}", papiExc.Error, papiExc.StackTrace));
                return false;
            }
        }

        public static bool End(byte boardnr)
        {
            try
            {
                _papi.End(boardnr);
                return true;
            }
            catch (CPapiException papiExc)
            {
                Console.WriteLine(string.Format("Papi Error: {0} {1}", papiExc.Error, papiExc.StackTrace));
                return false;
            }
        }



        /// Threads ----------------------------------------------------------------------------------

        // This method that will be called when the thread is started
        public static void ListenerThread()
        {
            Console.WriteLine("Listener Thread started");
            while (true)
            {
                // Receive service indication or service confirmation
                ServiceConfirmationIndication serviceConfInd = _papi.ReceiveServiceConfirmIndication(boardNumber);

                if (serviceConfInd != null)
                {
                    if (serviceConfInd.IsConInd) // ConIndReceived
                    {
                        // confirmation / indication received
                        Console.WriteLine("Confirmation/Indication received!");

                        if (serviceConfInd.ServiceDescriptor.Data.GetType() == typeof(C_DPS_INIT_SLAVE_CON))
                        {
                            if (serviceConfInd.ServiceDescriptor.Result == (short)ServiceResult.Pos)
                            {
                                Console.WriteLine("Result Pos");
                                if (((C_DPS_INIT_SLAVE_CON)serviceConfInd.ServiceDescriptor.Data).Status == (short)DPSServiceResult.OK)
                                {
                                    Console.WriteLine("Init Successful");
                                }
                            }
                            else
                            {
                                Console.WriteLine("Result Neg");
                            }
                            Console.WriteLine("Rec Status: " + ((C_DPS_INIT_SLAVE_CON)serviceConfInd.ServiceDescriptor.Data).Status.ToString());
                            Console.WriteLine("Rec RemMem: " + ((C_DPS_INIT_SLAVE_CON)serviceConfInd.ServiceDescriptor.Data).RemainingFrameMemory.ToString());
                        }

                        if (serviceConfInd.ServiceDescriptor.Data.GetType() == typeof(C_DPS_GET_STATUS_CON_IND))
                        {
                            Console.WriteLine("STATUS:");
                            Console.WriteLine("SlaveState: " + ((C_DPS_GET_STATUS_CON_IND)serviceConfInd.ServiceDescriptor.Data).SlaveState.ToString());
                            Console.WriteLine("SlaveAdd: " + ((C_DPS_GET_STATUS_CON_IND)serviceConfInd.ServiceDescriptor.Data).SlaveAdd.ToString());
                            Console.WriteLine("Status: " + ((C_DPS_GET_STATUS_CON_IND)serviceConfInd.ServiceDescriptor.Data).Status.ToString());
                            Console.WriteLine("NumInputs: " + ((C_DPS_GET_STATUS_CON_IND)serviceConfInd.ServiceDescriptor.Data).NumberInputs.ToString());
                            Console.WriteLine("NumOutputs: " + ((C_DPS_GET_STATUS_CON_IND)serviceConfInd.ServiceDescriptor.Data).NumberOutputs.ToString());
                            Console.WriteLine("MasterAdd: " + ((C_DPS_GET_STATUS_CON_IND)serviceConfInd.ServiceDescriptor.Data).MasterAdd.ToString());
                            Console.WriteLine("AwaitStartup: " + ((C_DPS_GET_STATUS_CON_IND)serviceConfInd.ServiceDescriptor.Data).AwaitStartupInputs.ToString());
                            Console.WriteLine("CfgAwaitResp: " + ((C_DPS_GET_STATUS_CON_IND)serviceConfInd.ServiceDescriptor.Data).CfgAwaitResponse.ToString());
                            Console.WriteLine("PrmAwaitResp: " + ((C_DPS_GET_STATUS_CON_IND)serviceConfInd.ServiceDescriptor.Data).PrmAwaitResponse.ToString());
                            Console.WriteLine("DiagState: " + ((C_DPS_GET_STATUS_CON_IND)serviceConfInd.ServiceDescriptor.Data).DiagState.ToString());
                        }
                    }
                    else
                    {
                        // no confirmation and no indication received
                      //  Console.WriteLine("no confirmation and no indication received!");
                    }
                }
            } // end while
        }
    }
}
